********************************************************************************
* Name          | PWEncryp
* Purpose       | Encrypts and decrypts passwords (or any string).
* Syntax        | expC = PWEncryp( <expC1> [[, <expL>] [, <expL2>]]] )
* Parameters    | <expC1>  tcPassWord
*               |          String to encrypt or decrypt.
*               | <expL>   tlEncrypt      Default = .F. = decrypt
*               |          Encrypt string?
*               | <expC2>  tcKeyWord      Default = 'StephenHeider'
*               |          Keyword to perform encryption and decryption.
* Returns       | Encrypted or decrypted string.  If tcPassWord is '', then
*               | '' is returned.
* Example       | IF PWEncryp(sy_users.cUserPW) == pcUserPW
* Requirements  | This routine should not be used where security is important.
*               | The routine uses simple character substitution.  It would
*               | not take very long for someone to break the keyword.
*               | Don't make the keyword upper case.  Otherwise, every space
*               | in the encrypted string will be the lower case letter of
*               | the keyword.
*               | When encrypting, any character with an ASC() value below 32
*               | is converted to a space - CHR(32).
* Tables Used   |
* Tables Changed|
* Notes         | String length is the same after encryption or decryption.
*               | There is no internal limit to the length of the password
*               | or keyword.  Any characters in the keyword that are longer
*               | than the length of the password are ignored.  Meaning, if
*               | the password length is 10 and the keyword length is 12, the
*               | last 2 characters of the keyword are ignored.
* Last Updated  | 08/03/95
* First Created | 07/11/95
* Author        | Stephen J. Heider 71572,3231
* FoxPro Version| DOS 2.6a
* Modifications | 07/11/95  John Didion 74001,151
*               |           Original creation.
*               | 08/03/95  Stephen J. Heider 71572,3231
*               |           Improved performance, decreased memory requirements,
*               |           documented code, implemented naming conventions.
********************************************************************************

PARAMETERS tcPassWord,tlEncrypt,tcKeyWord

* Set default keyword if not passed:
tcKeyWord = IIF(PARA() < 3, 'StephenHeider', tcKeyWord)

* Check for missing password:
IF LEN(tcPassWord) = 0
  RETURN ''
ENDIF

PRIVATE pcKeyWord,pcReturn,pnCounter,pnCharValue

* Pad keyword to the size of the password.
pcKeyWord = TRIM(PADR(tcKeyWord,LEN(tcPassWord)))
DO WHILE LEN(pcKeyWord) < LEN(tcPassWord)
  pcKeyWord = pcKeyWord + tcKeyWord
ENDDO

* Build return string:
pcReturn = ''
IF tlEncrypt
  * Encrypt string:
  FOR pnCounter = 1 TO LEN(tcPassWord)
    pnCharValue = MAX(32,ASC(SUBS(tcPassWord, pnCounter, 1))) + MAX(32,ASC(SUBS(pcKeyWord, pnCounter, 1)))
    pcReturn = pcReturn + CHR(IIF(pnCharValue > 255, ((pnCharValue - 255) + 31), pnCharValue))
  NEXT
ELSE
  * Decrypt string:
  FOR pnCounter = 1 TO LEN(tcPassWord)
    pnCharValue = ASC(SUBS(tcPassWord, pnCounter, 1)) - ASC(SUBS(pcKeyWord, pnCounter, 1))
    pcReturn = pcReturn + CHR(IIF(pnCharValue < 32, ((pnCharValue + 255) - 31), pnCharValue))
  NEXT
ENDIF

RETURN pcReturn